@@ -58,16 +58,16 @@ import rpy2.robjects as ro
 import rpy2.robjects.packages as rpacks
 from rpy2.robjects.lib import grdevices
 from rpy2.robjects.conversion import (Converter,
-                                      localconverter)
+                                      localconverter,
+                                      get_conversion)
 import warnings
-from rpy2.robjects.conversion import converter as template_converter
 
 # Try loading pandas and numpy, emitting a warning if either cannot be
 # loaded.
 try:
-    import numpy
+    import numpy  # type: ignore
     try:
-        import pandas
+        import pandas  # type: ignore
     except ImportError as ie:
         pandas = None
         warnings.warn('The Python package `pandas` is strongly '
@@ -86,24 +86,40 @@ except ImportError as ie:
 
 # IPython imports.
 
-from IPython.core import displaypub
-from IPython.core.magic import (Magics,
+from IPython.core import displaypub  # type: ignore
+from IPython.core.magic import (Magics,   # type: ignore
                                 magics_class,
                                 line_cell_magic,
                                 line_magic,
                                 needs_local_scope)
-from IPython.core.magic_arguments import (argument,
+from IPython.core.magic_arguments import (argument,  # type: ignore
                                           argument_group,
                                           magic_arguments,
                                           parse_argstring)
 
+template_converter = get_conversion()
 
-if numpy:
-    from rpy2.robjects import numpy2ri
-    template_converter += numpy2ri.converter
-    if pandas:
-        from rpy2.robjects import pandas2ri
-        template_converter += pandas2ri.converter
+
+def _get_ipython_template_converter(template_converter=template_converter,
+                                    numpy=numpy, pandas=pandas):
+    if numpy:
+        from rpy2.robjects import numpy2ri
+        template_converter += numpy2ri.converter
+        if pandas:
+            from rpy2.robjects import pandas2ri
+            template_converter += pandas2ri.converter
+    return template_converter
+
+
+def _get_converter(template_converter=template_converter):
+    return Converter('ipython conversion',
+                     template=template_converter)
+
+
+ipy_template_converter = _get_ipython_template_converter(template_converter,
+                                                         numpy=numpy,
+                                                         pandas=pandas)
+converter = _get_converter(template_converter=ipy_template_converter)
 
 
 def CELL_DISPLAY_DEFAULT(res, args):
@@ -130,20 +146,25 @@ class RInterpreterError(ri.embedded.RRuntimeError):
         return s
 
 
-converter = Converter('ipython conversion',
-                      template=template_converter)
-
-
 # The default conversion for lists is currently to make them an R list. That
 # has some advantages, but can be inconvenient (and, it's inconsistent with
 # the way python lists are automatically converted by numpy functions), so
 # for interactive use in the rmagic, we call unlist, which converts lists to
 # vectors **if the list was of uniform (atomic) type**.
-@converter.rpy2py.register(list)
-def rpy2py_list(obj):
+@converter.py2rpy.register(list)
+def py2rpy_list(obj):
     # simplify2array is a utility function, but nice for us
     # TODO: use an early binding of the R function
-    return ro.r.simplify2array(obj)
+    cv = ro.conversion.get_conversion()
+    robj = ri.ListSexpVector(
+            [cv.py2rpy(x) for x in obj]
+        )
+    res = ro.r.simplify2array(robj)
+    # The current default converter for the ipython rmagic
+    # might make `res` a numpy array. We need to ensure that
+    # a rpy2 objects is returned (issue #866).
+    res_rpy = cv.py2rpy(res)
+    return res_rpy
 
 
 # TODO: remove ?
@@ -187,11 +208,8 @@ class RMagics(Magics):
         """
         super(RMagics, self).__init__(shell)
         self.cache_display_data = cache_display_data
-
         self.Rstdout_cache = []
-
         self.converter = converter
-
         self.set_R_plotting_device(device)
 
     def set_R_plotting_device(self, device):
@@ -201,14 +219,11 @@ class RMagics(Magics):
         must be installed. Because Cairo forces "onefile=TRUE",
         it is not posible to include multiple plots per cell.
 
-        Parameters
-        ----------
-
-        device : ['png', 'X11', 'svg']
-            Device to be used for plotting.
-            Currently only "png" and "X11" are supported,
-            with 'png' and 'svg' being most useful in the notebook,
-            and 'X11' allowing interactive plots in the terminal.
+        :param device: ['png', 'X11', 'svg']
+          Device to be used for plotting.
+          Currently only "png" and "X11" are supported,
+          with 'png' and 'svg' being most useful in the notebook,
+          and 'X11' allowing interactive plots in the terminal.
 
         """
         device = device.strip()
@@ -780,28 +795,28 @@ class RMagics(Magics):
             print(e.stdout)
             if not e.stdout.endswith(e.err):
                 print(e.err)
-            if tmpd:
-                rmtree(tmpd)
-            return
+            raise e
         finally:
             if self.device in ['png', 'svg']:
                 ro.r('dev.off()')
+            if text_output:
+                # display_data.append(('RMagic.R', {'text/plain':text_output}))
+                displaypub.publish_display_data(
+                    data={'text/plain': text_output}, source='RMagic.R')
+            # publish the R images
+            if self.device in ['png', 'svg']:
+                display_data, md = self.publish_graphics(
+                    tmpd, args.isolate_svgs
+                )
 
-        if text_output:
-            # display_data.append(('RMagic.R', {'text/plain':text_output}))
-            displaypub.publish_display_data(
-                data={'text/plain': text_output}, source='RMagic.R')
-        # publish the R images
-        if self.device in ['png', 'svg']:
-            display_data, md = self.publish_graphics(tmpd, args.isolate_svgs)
-
-            for tag, disp_d in display_data:
-                displaypub.publish_display_data(data=disp_d, source=tag,
-                                                metadata=md)
+                for tag, disp_d in display_data:
+                    displaypub.publish_display_data(data=disp_d, source=tag,
+                                                    metadata=md)
 
             # kill the temporary directory - currently created only for "svg"
             # and "png" (else it's None)
-            rmtree(tmpd)
+            if tmpd:
+                rmtree(tmpd)
 
         if args.output:
             with localconverter(converter) as cv:
@@ -826,10 +841,10 @@ class RMagics(Magics):
 
 
 __doc__ = __doc__.format(
-                R_DOC=' '*8 + RMagics.R.__doc__,
-                RPUSH_DOC=' '*8 + RMagics.Rpush.__doc__,
-                RPULL_DOC=' '*8 + RMagics.Rpull.__doc__,
-                RGET_DOC=' '*8 + RMagics.Rget.__doc__
+                R_DOC='{0}{1}'.format(' '*8, RMagics.R.__doc__),
+                RPUSH_DOC='{0}{1}'.format(' '*8, RMagics.Rpush.__doc__),
+                RPULL_DOC='{0}{1}'.format(' '*8, RMagics.Rpull.__doc__),
+                RGET_DOC='{0}{1}'.format(' '*8, RMagics.Rget.__doc__)
 )
 
 
